package pps.core.space.utils;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateUtil;
import org.apache.commons.lang3.StringUtils;
import pps.cloud.base.service.IBasePowerLineCloudService;
import pps.cloud.base.service.IBasePriceStrategyCloudService;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineInput;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineOutput;
import pps.cloud.base.service.data.base_power_line.GetBasePowerLineInput;
import pps.cloud.base.service.data.base_power_line.GetBasePowerLineViewOutput;
import pps.cloud.base.service.data.base_price_strategy_detail.GetBasePriceStrategyDetailInput;
import pps.cloud.base.service.data.base_price_strategy_detail.GetBasePriceStrategyDetailOutput;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.*;
import pps.core.common.constant.BusinessConstant;
import pps.core.space.service.data.SpaceOptimizeDateDuration;
import pps.core.space.service.data.SpaceOptimizeDateOutput;
import xstartup.base.XContext;
import xstartup.data.XListResult;
import xstartup.data.XSingleResult;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 常用服务调用工具类
 *
 * @author ZWT
 * @date 2023/09/05
 */
public class ServiceUtil {

    /*-------------------------服务调用-------------------------*/

    /**
     * 通过字典类型查询字典
     *
     * @param context 上下文
     * @param alias   别名
     * @return {@link Map}<{@link String}, {@link String}>
     */
    public static Map<String, String> getDictMap(XContext context, String alias) {
        QuerySysDictionaryViewInput dictInput = new QuerySysDictionaryViewInput();
        dictInput.setAlias(alias);
        dictInput.setPage(1);
        dictInput.setLimit(999);
        SystemDictionaryService dictionaryService = context.getBean(SystemDictionaryService.class);
        XListResult<QuerySysDictionaryViewOutput> dictPageResult = dictionaryService.queryChildSysDictionarysByParentAlias(context, dictInput);
        dictPageResult.throwIfFail();
        return dictPageResult.getResult().stream()
                .collect(Collectors.toMap(QuerySysDictionaryViewOutput::getDicKey, QuerySysDictionaryViewOutput::getDicName));
    }

    /**
     * 通过组织id获取系统组织名称
     *
     * @param context 上下文
     * @param ouId    组织id
     * @return {@link String}
     */
    public static String getSysOrgNameById(XContext context, String ouId) {
        SysOrganizationCloudService orgCloudService = context.getBean(SysOrganizationCloudService.class);
        GetSysOrganizationViewInput viewInput = new GetSysOrganizationViewInput();
        viewInput.setId(ouId);
        XSingleResult<GetSysOrganizationViewOutput> organization = orgCloudService.getSysOrganizationById(context, viewInput);
        organization.throwIfFail();
        return Optional.ofNullable(organization.getResult())
                .map(GetSysOrganizationViewOutput::getOuName)
                .orElse(null);
    }

    /**
     * 通过组织机构ID集合获取组织名称
     *
     * @param context 上下文
     * @param ouIds   ou id
     * @return {@link Map}<{@link String}, {@link String}>
     */
    public static Map<String, String> getOuMapByOuIds(XContext context, List<String> ouIds) {
        GetOuListByOuIdsInput input = new GetOuListByOuIdsInput();
        input.setOuIdList(ouIds);
        SysOrganizationCloudService organizationCloudService = context.getBean(SysOrganizationCloudService.class);
        XListResult<GetOuListTreeOutput> ouListByOuIds = organizationCloudService.getOuListByOuIds(context, input);
        ouListByOuIds.throwIfFail();
        return ouListByOuIds.getResult().stream()
                .collect(Collectors.toMap(
                        GetOuListTreeOutput::getOuId, GetOuListTreeOutput::getOuName
                ));
    }

    /**
     * 通过ID查询线路名称
     *
     * @param context 上下文
     * @param lineId  线路id
     * @return {@link String}
     */
    public static String getPowerLineNameById(XContext context, String lineId) {
        IBasePowerLineCloudService lineService = context.getBean(IBasePowerLineCloudService.class);
        XSingleResult<GetBasePowerLineViewOutput> result = lineService.getBasePowerLineById(context, GetBasePowerLineInput.builder()
                .id(lineId)
                .build());
        result.throwIfFail();
        return Optional.ofNullable(result.getResult())
                .map(GetBasePowerLineViewOutput::getLineName)
                .orElse(null);
    }

    /**
     * 通过线路ID集合获取线路名称Map
     *
     * @param context 上下文
     * @param lineIds 线路ID
     * @return {@link Map}<{@link String}, {@link String}>
     */
    public static Map<String, String> getPowerLineNameMap(XContext context, List<String> lineIds) {
        IBasePowerLineCloudService lineService = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLineOutput> result = lineService.getBasePowerLineList(context, DynamicQueryBasePowerLineInput.builder()
                .lineIds(lineIds)
                .build());
        result.throwIfFail();
        return result.getResult().stream()
                .collect(
                        Collectors.toMap(DynamicQueryBasePowerLineOutput::getId, DynamicQueryBasePowerLineOutput::getLineName)
                );
    }

    /**
     * 条件查询策略配置详情列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link List}<{@link GetBasePriceStrategyDetailOutput}>
     */
    public static List<GetBasePriceStrategyDetailOutput> getStrategyDetailList(XContext context, GetBasePriceStrategyDetailInput input) {
        IBasePriceStrategyCloudService service = context.getBean(IBasePriceStrategyCloudService.class);
        XListResult<GetBasePriceStrategyDetailOutput> result = service.queryStrategyDetailList(context, input);
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 获取优化日期列表
     *
     * @param begin 开始
     * @param end   终止
     * @return {@link List}<{@link SpaceOptimizeDateOutput}>
     */
    public static List<SpaceOptimizeDateOutput> getOptimizeDateList(Date begin, Date end) {
        return DateUtil.rangeToList(
                        DateUtil.beginOfDay(begin),
                        DateUtil.endOfDay(end),
                        DateField.DAY_OF_YEAR).stream()
                .map(r -> {
                    SpaceOptimizeDateOutput spaceOptimizeDateOutput = new SpaceOptimizeDateOutput();
                    spaceOptimizeDateOutput.setDateTime(r);
                    spaceOptimizeDateOutput.setOptimizeCode(BusinessConstant.ZERO);
                    spaceOptimizeDateOutput.setOptimizeValue("已优化");
                    return spaceOptimizeDateOutput;
                }).collect(Collectors.toList());
    }

    /**
     * 封装全天时间段
     *
     * @param optimizeList 优化列表
     * @return {@link List}<{@link SpaceOptimizeDateDuration}>
     */
    public static List<SpaceOptimizeDateDuration> getAllDateDuration(List<SpaceOptimizeDateDuration> optimizeList) {
        List<SpaceOptimizeDateDuration> allList = new ArrayList<>(16);
        if (CollUtil.isEmpty(optimizeList)) {
            allList.add(
                    SpaceOptimizeDateDuration.builder()
                            .openWellTime("00:00")
                            .closeWellTime("23:59")
                            .isOpen(0)
                            .build()
            );
            return allList;
        }
        int size = optimizeList.size();
        for (int i = 0; i < size; i++) {
            SpaceOptimizeDateDuration optimize = optimizeList.get(i);
            String openWellTime = optimize.getOpenWellTime();
            String closeWellTime = optimize.getCloseWellTime();
            optimize.setIsOpen(1);
            if (i == 0) {
                //如果开始时间不是从00:00开始,初始化开始时间为关井时间
                if (!StringUtils.equals("00:00", openWellTime)) {
                    allList.add(
                            SpaceOptimizeDateDuration.builder()
                                    .openWellTime("00:00")
                                    .closeWellTime(openWellTime)
                                    .isOpen(0)
                                    .build()
                    );
                }
            }
            //将当前时段添加到新集合
            allList.add(optimize);
            //判断当前时段到下一时段间是否有关井时段
            int next = i + 1;
            int last = size - 1;
            if (next <= last) {
                SpaceOptimizeDateDuration nextOptimize = optimizeList.get(next);
                String nextOpenWellTime = nextOptimize.getOpenWellTime();
                if (!StringUtils.equals(closeWellTime, nextOpenWellTime)) {
                    allList.add(
                            SpaceOptimizeDateDuration.builder()
                                    .openWellTime(closeWellTime)
                                    .closeWellTime(nextOpenWellTime)
                                    .isOpen(0)
                                    .build()
                    );
                }
            }
            if (i == last) {
                //如果结束时间不是24:00开始,初始化最终关井时间
                if (!StringUtils.equals("23:59", closeWellTime)) {
                    allList.add(
                            SpaceOptimizeDateDuration.builder()
                                    .openWellTime(closeWellTime)
                                    .closeWellTime("23:59")
                                    .isOpen(0)
                                    .build()
                    );
                }
            }
        }
        return allList;
    }
}